import logging

from django.conf import settings
from django.core.urlresolvers import resolve, Resolver404
from django.http import Http404, HttpResponsePermanentRedirect, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.utils import timezone
from django.utils.cache import add_never_cache_headers
from django.views.generic import ListView, DetailView, View

from bulbs.content.custom_search import custom_search_model
from bulbs.content.models import Content, ObfuscatedUrlInfo

logger = logging.getLogger(__name__)


class ContentListView(ListView):
    model = Content

    feature_types = None
    tags = None
    types = None
    published = None
    before = None
    after = None

    allow_empty = True
    paginate_by = 20
    context_object_name = "content_list"
    template_name = None

    def get_queryset(self):
        search_kwargs = {}
        if "tags" in self.request.GET:
            search_kwargs["tags"] = self.request.GET.getlist("tags", [])
        elif "tag" in self.request.GET:
            search_kwargs["tags"] = self.request.GET.getlist("tag", [])

        if "tags" in self.kwargs:
            tags = self.kwargs["tags"]
            if not isinstance(tags, list):
                tags = [tags]
            search_kwargs["tags"] = tags
        try:
            if self.tags > 0:
                search_kwargs["tags"] = self.tags
        except TypeError:
            pass

        if "types" in self.request.GET:
            search_kwargs["types"] = self.request.GET.getlist("types", [])
        elif "type" in self.request.GET:
            search_kwargs["types"] = self.request.GET.getlist("type", [])

        if "types" in self.kwargs:
            search_kwargs["types"] = self.kwargs["types"]

        try:
            if self.types > 0:
                search_kwargs["types"] = self.types
        except TypeError:
            pass

        if "feature_types" in self.request.GET:
            search_kwargs["feature_types"] = self.request.GET.getlist("feature_types", [])
        elif "feature_type" in self.request.GET:
            search_kwargs["feature_types"] = self.request.GET.getlist("feature_type", [])

        if "feature_types" in self.kwargs:
            search_kwargs["feature_types"] = self.kwargs["feature_types"]
        try:
            if self.feature_types > 0:
                search_kwargs["feature_types"] = self.feature_types
        except TypeError:
            pass

        if "published" in self.kwargs:
            search_kwargs["published"] = self.kwargs["published"]
        if self.published:
            search_kwargs["published"] = self.published

        if "before" in self.request.GET:
            search_kwargs["before"] = self.request.GET["before"]
        if "before" in self.kwargs:
            search_kwargs["before"] = self.kwargs["before"]
        if self.before:
            search_kwargs["before"] = self.before

        if "after" in self.request.GET:
            search_kwargs["after"] = self.request.GET["after"]
        if "after" in self.kwargs:
            search_kwargs["after"] = self.kwargs["after"]
        if self.after:
            search_kwargs["after"] = self.after

        if "q" in self.request.GET:
            search_kwargs["query"] = self.request.GET["q"]

        return self.model.search_objects.search(**search_kwargs)


class PaginatedMixin(object):

    def get_context_data(self, **kwargs):
        context = super(PaginatedMixin, self).get_context_data(**kwargs)

        params = self.request.GET.copy()
        page = context.get("page_obj")
        if page:
            if page.has_next():
                params["page"] = page.next_page_number()
                context["next_url"] = u"{0}?{1}".format(self.request.path, params.urlencode())
            if page.has_previous():
                params["page"] = page.previous_page_number()
                context["previous_url"] = u"{0}?{1}".format(self.request.path, params.urlencode())

        return context


class ContentCustomSearchListView(ListView):
    model = Content
    paginate_by = 20
    context_object_name = "content_list"
    is_preview = False
    is_published = True
    field_map = {
        "feature-type": "feature_type.slug",
        "tag": "tags.slug",
        "content-type": "_type"
    }

    def get_queryset(self):
        query = self.get_search_query()
        return self.get_custom_search_queryset(query)

    def get_search_query(self):
        return {}

    def get_custom_search_queryset(self, query):
        qs = custom_search_model(
            self.model, query, preview=self.is_preview,
            published=self.is_published, field_map=self.field_map
        )
        return qs


class BaseContentDetailView(DetailView):
    """Should be used as the base for all content detail views."""

    model = Content
    context_object_name = "content"
    redirect_correct_path = True  # By default, we'll redirect the user to the proper URL

    def show_published_only(self):
        return bool("full_preview" not in self.request.GET)

    def get(self, request, *args, **kwargs):
        """Override default get function to use token if there is one to retrieve object. If a
        subclass should use their own GET implementation, token_from_kwargs should be called if
        that detail view should be accessible via token."""

        self.object = self.get_object()

        allow_anonymous = kwargs.get("allow_anonymous", False)

        # We only want to redirect is that setting is true, and we're not allowing anonymous users
        if self.redirect_correct_path and not allow_anonymous:

            # Also we obviously only want to redirect if the URL is wrong
            if self.request.path != self.object.get_absolute_url():
                return HttpResponsePermanentRedirect(self.object.get_absolute_url())

        context = self.get_context_data(object=self.object)
        response = self.render_to_response(context)

        # If we have an unpublished article....
        if self.object.published is None or self.object.published > timezone.now():

            # And the user doesn't have permission to view this
            if not request.user.is_staff and not allow_anonymous:
                redirect_unpublished = getattr(settings, "REDIRECT_UNPUBLISHED_TO_LOGIN", True)
                if not request.user.is_authenticated() and redirect_unpublished:
                    next_url = self.object.get_absolute_url()
                    response = HttpResponseRedirect("{}?next={}".format(settings.LOGIN_URL, next_url))
                else:
                    raise Http404

            # Never cache unpublished articles
            add_never_cache_headers(response)
        else:
            response["Vary"] = "Accept-Encoding"

        return response


class UnpublishedContentView(View):
    def dispatch(self, request, *args, **kwargs):

        token = kwargs.get("token", None)
        info = get_object_or_404(ObfuscatedUrlInfo, url_uuid=token)

        now = timezone.now()
        if info.create_date >= now or now > info.expire_date:
            # date doesn't match, don't let user access
            raise Http404("No content found matching UUID")

        try:
            view, args, kwargs = resolve(info.content.get_absolute_url())
        except Resolver404:
            raise Http404("No content found matching UUID")

        kwargs["request"] = request
        kwargs["allow_anonymous"] = True
        return view(*args, **kwargs)


unpublished = UnpublishedContentView.as_view()
content_list = ContentListView.as_view()
content_custom_search_list = ContentCustomSearchListView.as_view()



from django.contrib.auth import get_user_model

from rest_framework import serializers

from .models import Contribution


contributor_cls = get_user_model()


class ContributionCSVSerializer(serializers.ModelSerializer):

    class Meta:
        model = Contribution

    def to_representation(self, obj):
        full_name = obj.contributor.get_full_name()
        data = {
            'id': obj.content.id,
            'first_name': obj.contributor.first_name,
            'last_name': obj.contributor.last_name,
            'title': obj.content.title,
            'feature_type': obj.content.feature_type,
            'publish_date': obj.content.published,
            'rate': obj.get_pay,
            'payroll_name': full_name
        }
        profile = getattr(obj.contributor, 'freelanceprofile', None)
        if profile:
            payroll_name = getattr(profile, 'payroll_name', None)
            if payroll_name:
                data['payroll_name'] = payroll_name
        return data



from django.template import RequestContext
from django.utils.timezone import now
from django.views.decorators.cache import cache_control

from bulbs.content.views import ContentListView
from bulbs.special_coverage.models import SpecialCoverage


class RSSView(ContentListView):
    """Really simply, this syndicates Content."""
    template_name = "feeds/rss.xml"
    paginate_by = 20
    feed_title = "RSS Feed"
    utm_params = "utm_medium=RSS&amp;utm_campaign=feeds"

    def get_template_names(self):
        return ["feeds/rss.xml", "feeds/_rss.xml"]

    @cache_control(max_age=600)
    def get(self, request, *args, **kwargs):
        response = super(RSSView, self).get(request, *args, **kwargs)
        response["Content-Type"] = "application/rss+xml"
        return response

    def get_context_data(self, *args, **kwargs):
        context = super(RSSView, self).get_context_data(*args, **kwargs)
        context["full"] = (self.request.GET.get("full", "false").lower() == "true")
        context["images"] = (self.request.GET.get("images", "false").lower() == "true")
        context["build_date"] = now()
        context["title"] = self.feed_title
        context["feed_url"] = self.request.build_absolute_uri()
        context["search_url"] = self.request.build_absolute_uri(
            u"/search?%s" % self.request.META["QUERY_STRING"])

        # OK, so this is kinda brutal. Stay with me here.
        for content in context["page_obj"].object_list:
            feed_path = content.get_absolute_url() + "?" + self.utm_params
            content.feed_url = self.request.build_absolute_uri(feed_path)

        return RequestContext(self.request, context)


class SpecialCoverageRSSView(RSSView):
    """Really simply, this syndicates Content."""
    feed_title = "Special Coverage RSS Feed"

    def get_queryset(self):
        sc_id = self.request.GET.get("special_coverage_id")
        sc_slug = self.request.GET.get("special_coverage_slug")

        if sc_id:
            sc = SpecialCoverage.objects.get(id=sc_id)
        elif sc_slug:
            sc = SpecialCoverage.objects.get(slug=sc_slug)
        else:
            return self.model.objects.none()

        return sc.get_content()[:self.paginate_by]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('promotion', '0002_content_list_to_pzone'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='pzone',
            options={'ordering': ['name']},
        ),
        migrations.AlterModelOptions(
            name='pzonehistory',
            options={'ordering': ['-date']},
        ),
        migrations.AlterModelOptions(
            name='pzoneoperation',
            options={'ordering': ['when', 'id']},
        ),
    ]



from rest_framework import permissions


class HasPermissionOrIsAuthor(permissions.BasePermission):
    authors_field = "authors"
    require_staff = True
    permissions = []  # permissions which would permit a non-author to do this
    author_permissions = []  # permissions the author needs in order to be permitted
    protected_methods = []

    def has_object_permission(self, request, view, obj):
        """determines if requesting user has permissions for the object

        :param request: WSGI request object - where we get the user from
        :param view: the view calling for permission
        :param obj: the object in question
        :return: `bool`
        """
        # Give permission if we're not protecting this method
        if self.protected_methods and not request.method in self.protected_methods:
            return True

        user = getattr(request, "user", None)

        if not user or user.is_anonymous():
            return False

        if self.require_staff and not user.is_staff:
            return False

        # if they have higher-level privileges we can return true right now
        if user.has_perms(self.permissions):
            return True

        # no? ok maybe they're the author and have appropriate author permissions.
        authors_field = getattr(obj, self.authors_field, None)

        if not authors_field:
            return False

        if self.author_permissions and not user.has_perms(self.author_permissions):
            return False

        return user in authors_field.all()


class CanEditContent(HasPermissionOrIsAuthor):
    """
    extends `HasPermissionOrIsAuthor` and protects PUT method
    """

    permissions = ["content.change_content"]
    protected_methods = ["PUT"]


class CanPromoteContent(HasPermissionOrIsAuthor):
    """
    extends `HasPermissionOrIsAuthor` and ensures user has promotion rights
    """

    permissions = ["content.promote_content"]


class CanPublishContent(HasPermissionOrIsAuthor):
    """
    extends `HasPermissionOrIsAuthor` and ensures user has publishing rights
    """

    permissions = ["content.publish_content"]
    author_permissions = ["content.publish_own_content"]


class CanEditCmsNotifications(permissions.BasePermission):
    """
    permission class to handle CMS rights
    """

    def has_permission(self, request, view):
        """If method is GET, user can access, if method is PUT or POST user must be a superuser.
        """
        has_permission = False
        if request.method == "GET" \
                or request.method in ["PUT", "POST", "DELETE"] \
                and request.user and request.user.is_superuser:
            has_permission = True
        return has_permission



from django.db import models


class CmsNotification(models.Model):
    """A CMS notification.
    """

    title = models.CharField(max_length=110)
    body = models.TextField(blank=True)
    post_date = models.DateTimeField()
    notify_end_date = models.DateTimeField()

    class Meta:
        ordering = ['-id']



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'CmsNotification'
        db.create_table(u'cms_notifications_cmsnotification', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=110)),
            ('body', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('post_date', self.gf('django.db.models.fields.DateTimeField')()),
            ('notify_end_date', self.gf('django.db.models.fields.DateTimeField')()),
        ))
        db.send_create_signal(u'cms_notifications', ['CmsNotification'])


    def backwards(self, orm):
        # Deleting model 'CmsNotification'
        db.delete_table(u'cms_notifications_cmsnotification')


    models = {
        u'cms_notifications.cmsnotification': {
            'Meta': {'ordering': "['-id']", 'object_name': 'CmsNotification'},
            'body': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'notify_end_date': ('django.db.models.fields.DateTimeField', [], {}),
            'post_date': ('django.db.models.fields.DateTimeField', [], {}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '110'})
        }
    }

    complete_apps = ['cms_notifications']


from django.conf.urls import url, patterns

from .views import RSSView, SpecialCoverageRSSView


urlpatterns = patterns("",
    url(r"^rss", RSSView.as_view(), name="rss-feed"),  # noqa
    url(r"^sc-rss", SpecialCoverageRSSView.as_view(), name="sc-rss-feed")  # noqa
)



import operator

from django import template
from django.template.base import parse_bits, Variable, VariableDoesNotExist
from django.template.defaulttags import ForNode

from bulbs.promotion.models import PZone


register = template.Library()


class PZoneSequence(object):

    def __init__(self, pzone_name, slice_string=None, apply=True):
        self.pzone_name = pzone_name
        if slice_string:
            bits = []
            for x in slice_string.split(':'):
                if len(x) == 0:
                    bits.append(None)
                else:
                    bits.append(int(x))
            self.slice = slice(*bits)
        else:
            self.slice = None

        self.apply = apply

    def resolve(self, context, ignore_failures=False):
        
        try:
            when = Variable("pzone_preview").resolve(context)
            pzone = PZone.objects.preview(name=self.pzone_name, when=when)
        except VariableDoesNotExist:
            pzone = PZone.objects.applied(name=self.pzone_name)
        if self.slice:
            return pzone[self.slice]
        else:
            return pzone


@register.tag('forpzone')
def do_pzone(parser, token):
    """

    {% forpzone "homepage" slice=":3" %}
        <h1>
            <a href="{{ content.get_absolute_url }}">{{ content.title }}</a>
        </h1>
        <span>{{ content.description }}</span>
    {% endforpzone %}

    """

    bits = token.split_contents()

    if len(bits) < 2:
        raise TemplateSyntaxError("'pzone' statements should have at least two"
                                  " words: %s" % token.contents)

    nodelist_loop = parser.parse(("endforpzone","empty"))
    token = parser.next_token()
    if token.contents == 'empty':
        nodelist_empty = parser.parse(('endfor',))
        parser.delete_first_token()
    else:
        nodelist_empty = None

    params = ["slice", "name", "apply"]
    args, kwargs = parse_bits(parser, bits, params, None, None, [], False, "forpzone")
    
    pzone_name = kwargs["name"].resolve({})

    slice_string = None
    if "slice" in kwargs:
        slice_string = kwargs["slice"].resolve({})
        slice_bits = []
        for x in slice_string.split(':'):
            if len(x) == 0:
                slice_bits.append(None)
            else:
                slice_bits.append(int(x))
        slice_object = slice(*slice_bits)

    apply = False
    if "apply" in kwargs:
        apply = kwargs["apply"].resolve({})

    sequence = PZoneSequence(pzone_name, slice_string=slice_string, apply=apply)

    loopvars = ["content"]
    is_reversed = False

    return ForNode(loopvars, sequence, is_reversed, nodelist_loop, nodelist_empty)